@@ -1,10 +1,9 @@
-from typing import Optional
+from typing import Any, Optional
 
 from mautrix.bridge import Bridge
 from mautrix.bridge.state_store.asyncpg import PgBridgeStateStore
 from mautrix.types import RoomID, UserID
 from mautrix.util.async_db import Database
-from mautrix.util.bridge_state import BridgeState, BridgeStateEvent
 
 from . import commands  # noqa: F401
 from .config import Config
@@ -82,13 +81,6 @@ class LinkedInBridge(Bridge):
         await self.state_store.upgrade_table.upgrade(self.db.pool)
         if self.matrix.e2ee and self.matrix.e2ee.crypto_db:
             self.matrix.e2ee.crypto_db.override_pool(self.db.pool)
-
-        # Let the API server know that we are alive.
-        state = BridgeState(state_event=BridgeStateEvent.UNCONFIGURED)
-        await state.send(
-            self.config["homeserver.status_endpoint"], self.az.as_token, self.log
-        )
-
         self.add_startup_actions(User.init_cls(self))
         self.add_startup_actions(Puppet.init_cls(self))
         Portal.init_cls(self)
@@ -121,6 +113,19 @@ class LinkedInBridge(Bridge):
     def is_bridge_ghost(self, user_id: UserID) -> bool:
         return bool(Puppet.get_id_from_mxid(user_id))
 
+    async def count_logged_in_users(self) -> int:
+        return len(
+            [user for user in User.by_li_member_urn.values() if user.li_member_urn]
+        )
+
+    async def manhole_global_namespace(self, user_id: UserID) -> dict[str, Any]:
+        return {
+            **await super().manhole_global_namespace(user_id),
+            "User": User,
+            "Portal": Portal,
+            "Puppet": Puppet,
+        }
+
 
 def main():
     LinkedInBridge().run()
