@@ -1,5 +1,5 @@
 from datetime import datetime
-from typing import cast, List, Optional
+from typing import cast, Optional
 
 from asyncpg import Record
 from attr import dataclass
@@ -56,7 +56,7 @@ class Message(Model):
         cls,
         li_message_urn: URN,
         li_receiver_urn: URN,
-    ) -> List["Message"]:
+    ) -> list["Message"]:
         query = Message.select_constructor("li_message_urn=$1 AND li_receiver_urn=$2")
         rows = await cls.db.fetch(
             query,
@@ -84,7 +84,7 @@ class Message(Model):
         return cls._from_row(row)
 
     @classmethod
-    async def delete_all_by_room(cls, room_id: RoomID) -> None:
+    async def delete_all_by_room(cls, room_id: RoomID):
         await cls.db.execute("DELETE FROM message WHERE mx_room=$1", room_id)
 
     @classmethod
@@ -111,7 +111,7 @@ class Message(Model):
         )
         return cls._from_row(row)
 
-    async def insert(self) -> None:
+    async def insert(self):
         query = Message.insert_constructor()
         await self.db.execute(
             query,
@@ -133,9 +133,9 @@ class Message(Model):
         li_sender_urn: URN,
         li_receiver_urn: URN,
         timestamp: datetime,
-        event_ids: List[EventID],
+        event_ids: list[EventID],
         mx_room: RoomID,
-    ) -> None:
+    ):
         if not event_ids:
             return
 
@@ -159,7 +159,7 @@ class Message(Model):
                 columns=cls._field_list,
             )
 
-    async def delete(self) -> None:
+    async def delete(self):
         q = """
             DELETE FROM message
              WHERE li_message_urn=$1
