@@ -1,5 +1,4 @@
 import asyncio
-import inspect
 import time
 from asyncio.futures import Future
 from datetime import datetime
@@ -12,12 +11,14 @@ from typing import (
     TYPE_CHECKING,
 )
 
+import aiohttp
 from linkedin_messaging import LinkedInMessaging, URN
 from linkedin_messaging.api_objects import (
     Conversation,
     ConversationEvent,
     ReactionSummary,
     RealTimeEventStreamEvent,
+    UserProfileResponse,
 )
 from mautrix.bridge import async_getter_lock, BaseUser, BridgeState
 from mautrix.errors import MNotFound
@@ -48,6 +49,8 @@ class User(DBUser, BaseUser):
     shutdown: bool = False
     config: Config
 
+    user_profile_cache: Optional[UserProfileResponse] = None
+
     by_mxid: dict[UserID, "User"] = {}
     by_li_member_urn: dict[URN, "User"] = {}
 
@@ -190,14 +193,13 @@ class User(DBUser, BaseUser):
             return True
         if not self.client or not await self.client.logged_in():
             return False
-        await self.push_bridge_state(BridgeStateEvent.CONNECTING)
 
-        if (
-            mp := (await self.client.get_user_profile()).mini_profile
-        ) and mp.entity_urn:
+        self.user_profile_cache = await self.client.get_user_profile()
+        if (mp := self.user_profile_cache.mini_profile) and mp.entity_urn:
             self.li_member_urn = mp.entity_urn
         else:
             return False
+        await self.push_bridge_state(BridgeStateEvent.CONNECTING)
 
         self.log.info("Loaded session successfully")
         self._track_metric(METRIC_LOGGED_IN, True)
@@ -224,18 +226,17 @@ class User(DBUser, BaseUser):
             except Exception:
                 self.log.exception("Exception checking login status")
                 self._is_logged_in = False
+                self.user_profile_cache = None
         return self._is_logged_in or False
 
     async def on_logged_in(self, client: LinkedInMessaging):
-        await self.push_bridge_state(BridgeStateEvent.CONNECTING)
         self.client = client
-        if (
-            mp := (await self.client.get_user_profile()).mini_profile
-        ) and mp.entity_urn:
+        self.user_profile_cache = await self.client.get_user_profile()
+        if (mp := self.user_profile_cache.mini_profile) and mp.entity_urn:
             self.li_member_urn = mp.entity_urn
         else:
-            await self.push_bridge_state(BridgeStateEvent.BAD_CREDENTIALS)
             raise Exception("No mini_profile.entity_urn on the user profile!")
+        await self.push_bridge_state(BridgeStateEvent.CONNECTING)
         await self.save()
         self.stop_listen()
         asyncio.create_task(self.post_login())
@@ -251,15 +252,15 @@ class User(DBUser, BaseUser):
                 self.log.info("Automatically enabling custom puppet")
                 await puppet.switch_mxid(access_token="auto", mxid=self.mxid)
         except Exception:
+            self.user_profile_cache = None
             self.log.exception("Failed to automatically enable custom puppet")
         await self.sync_threads()
         self.start_listen()
 
     async def logout(self):
         self.log.info("Logging out")
-        if self.listen_task:
-            self.log.info("Cancelling the listen task.")
-            self.listen_task.cancel()
+        self._is_logged_in = False
+        self.stop_listen()
         if self.client:
             self.log.info("Logging out the client.")
             await self.client.logout()
@@ -274,10 +275,12 @@ class User(DBUser, BaseUser):
             except KeyError:
                 pass
         self._track_metric(METRIC_LOGGED_IN, True)
-        self._is_logged_in = False
         self.client = None
+        self.user_profile_cache = None
         self.li_member_urn = None
         self.notice_room = None
+        self._is_logged_in = False
+        self.stop_listen()
         await self.save()
 
     # endregion
@@ -328,6 +331,7 @@ class User(DBUser, BaseUser):
                 try:
                     await self._sync_thread(conversation)
                 except Exception:
+                    self.user_profile_cache = None
                     self.log.exception(
                         f"Failed to sync thread {conversation.entity_urn}"
                     )
@@ -415,18 +419,20 @@ class User(DBUser, BaseUser):
         user = await User.get_by_li_member_urn(self.li_member_urn)
         if user and user.client:
             try:
-                if mp := (await user.client.get_user_profile()).mini_profile:
+                user_profile = user.user_profile_cache
+                if user_profile is not None:
+                    self.log.debug("Cache hit on user_profile_cache")
+                user_profile = user_profile or await user.client.get_user_profile()
+                if mp := user_profile.mini_profile:
                     state.remote_name = " ".join(
                         n for n in [mp.first_name, mp.last_name] if n
                     )
             except Exception:
+                self.user_profile_cache = None
                 pass
 
     def stop_listen(self):
         self.log.info("Stopping the listener.")
-        curframe = inspect.currentframe()
-        callstack = "\n".join([str(f) for f in inspect.getouterframes(curframe, 10)])
-        self.log.info(f"Call stack:\n{callstack}")
         if self.listen_task:
             self.log.info("Cancelling the listen task.")
             self.listen_task.cancel()
@@ -435,7 +441,7 @@ class User(DBUser, BaseUser):
     def on_listen_task_end(self, future: Future):
         if future.cancelled():
             self.log.info("Listener task cancelled")
-        if not self.shutdown:
+        if self.client and self._is_logged_in and not self.shutdown:
             self.start_listen()
 
     listener_event_handlers_created: bool = False
@@ -481,11 +487,28 @@ class User(DBUser, BaseUser):
 
     async def handle_linkedin_listener_error(self, error: Exception):
         self._track_metric(METRIC_CONNECTED, False)
-        await self.push_bridge_state(
-            BridgeStateEvent.TRANSIENT_DISCONNECT,
-            message=str(error),
-        )
         self._prev_connected_bridge_state = -600
+        self.user_profile_cache = None
+
+        if isinstance(error, aiohttp.client.TooManyRedirects):
+            # This means that the user's session is borked (the redirects mean it's
+            # trying to redirect to the login page).
+            await self.push_bridge_state(
+                BridgeStateEvent.BAD_CREDENTIALS,
+                message=f"TooManyRedirects: {error}",
+            )
+            if self.listen_task:
+                self.client = None
+                self.user_profile_cache = None
+                self.li_member_urn = None
+                self._is_logged_in = False
+                self._prev_connected_bridge_state = -600
+                self.stop_listen()
+        else:
+            await self.push_bridge_state(
+                BridgeStateEvent.TRANSIENT_DISCONNECT,
+                message=str(error),
+            )
 
     async def handle_linkedin_event(self, event: RealTimeEventStreamEvent):
         assert self.client
